#AutoIt3Wrapper_UseX64=n
#include "Bass.au3"
#include "BassExt.au3"
#include <GDIPlus.au3>
#include "GDIP.au3"
#include <Color.au3>

Global $iWidth = 480
Global $iHeight = 480
Global $fTransform = -0.05

Global $aColor[3] = [0, 0, 0], $iColorStep = 0

Opt("GUIOnEventMode", 1)

$sFile = FileOpenDialog("Open...", "", "playable formats (*.MP3;*.MP2;*.MP1;*.OGG;*.WAV;*.AIFF;*.AIF)")
___Debug($sFile = "", $sFile)

$hGui = GUICreate("FFT", $iWidth, $iHeight)
GUISetOnEvent(-3, "_EXIT")
GUISetState()

_GDIPlus_Startup()
$hGraphics = _GDIPlus_GraphicsCreateFromHWND($hGui)
$hBmpBuffer = _GDIPlus_BitmapCreateFromGraphics($iWidth, $iHeight, $hGraphics)
$hGfxBuffer = _GDIPlus_ImageGetGraphicsContext($hBmpBuffer)
_GDIPlus_GraphicsSetSmoothingMode($hGfxBuffer, 2)
$hPen = _GDIPlus_PenCreate(0xFF000000)
$hBrush = _GDIPlus_BrushCreateSolid(0xFF00AAFF)
$hBrushTrans = _GDIPlus_BrushCreateSolid(0x11000000)
$hBmpScale = _GDIPlus_BitmapCreateFromGraphics($iWidth, $iHeight, $hGraphics)
$hGfxScale = _GDIPlus_ImageGetGraphicsContext($hBmpScale)
_GDIPlus_GraphicsClear($hGfxBuffer, 0xFF000000)

_GDIPlus_GraphicsTranslateTransform($hGfxScale, -($iWidth / 2 * $fTransform), -($iHeight / 8 * $fTransform))
_GDIPlus_GraphicsScaleTransform($hGfxScale, 1 + $fTransform, 1 + $fTransform)
_GDIPlus_GraphicsSetInterpolationMode($hGfxScale, 0)

_BASS_Startup(@ScriptDir & "\bass.dll")
___Debug(@error, "load bass.dll")

_BASS_EXT_Startup(@ScriptDir & "\bassExt.dll")
___Debug(@error, "load bassext.dll")

_BASS_Init(0, -1, 44100, 0, "")
___Debug(@error, "initialize bass")

$hStream = _BASS_StreamCreateFile(False, $sFile, 0, 0, $BASS_SAMPLE_FLOAT)
___Debug(@error, "create stream from file")

$aFFT = _BASS_EXT_CreateFFT(47, 0, 0, $iWidth, $iHeight, 0, True)

_BASS_ChannelPlay($hStream, True)
___Debug(@error, "Start playing stream")

GUIRegisterMsg(0x000F, "WM_PAINT")
GUIRegisterMsg(0x0014, "WM_ERASEBKGND")

$iTimer = TimerInit()
While _BASS_ChannelIsActive($hStream)
	If TimerDiff($iTimer) > 25 Then
		$iTimer = TimerInit()
		_BrushSetColor($hBrush, $iColorStep, $aColor, 1)
		_GDIPlus_GraphicsClear($hGfxBuffer, 0xFF000000)
		_GDIPlus_GraphicsDrawImage($hGfxBuffer, $hBmpScale, 0, 0)
		_BASS_EXT_ChannelGetFFT($hStream, $aFFT, 6)
		If Not @error Then
			DllCall($ghGDIPDll, "int", "GdipFillPolygon", "handle", $hGfxBuffer, "handle", $hBrush, "ptr", $aFFT[0], "int", $aFFT[1], "int", "FillModeAlternate")
			DllCall($ghGDIPDll, "int", "GdipDrawPolygon", "handle", $hGfxBuffer, "handle", $hPen, "ptr", $aFFT[0], "int", $aFFT[1])
		EndIf
		_GDIPlus_GraphicsDrawImage($hGfxScale, $hBmpBuffer, 0, 0)
		_GDIPlus_GraphicsFillRect($hGfxScale, 0, 0, 480, 480, $hBrushTrans)
		_GDIPlus_GraphicsDrawImage($hGraphics, $hBmpBuffer, 0, 0)
	EndIf
WEnd

_Exit()

Func _BrushSetColor($hBrush, ByRef $iStep, ByRef $aCol, $iCnt = 1)
	$iStep += $iCnt
	Switch $iStep
		Case 1 To 255 ; Rot
			$aCol[0] += $iCnt
			$aCol[1] -= $iCnt
			$aCol[2] -= $iCnt
		Case 256 To 511 ; Grn
			$aCol[0] -= $iCnt
			$aCol[1] += $iCnt
			$aCol[2] -= $iCnt
		Case 512 To 767 ; Blau
			$aCol[0] -= $iCnt
			$aCol[1] -= $iCnt
			$aCol[2] += $iCnt
		Case Else
			$iStep = 0
	EndSwitch
	For $i = 0 To 2
		If $aCol[$i] < 0 Then $aCol[$i] = 0
		If $aCol[$i] > 255 Then $aCol[$i] = 255
	Next
	_GDIPlus_BrushSetSolidColor($hBrush, "0xFF" & Hex(_ColorSetRGB($aCol), 6))
EndFunc   ;==>_BrushSetColor

Func _Exit()
	_BASS_StreamFree($hStream)
	___Debug(@error, "free stream")

	_BASS_Free()
	___Debug(@error, "free bass")

	_GDIPlus_PenDispose($hPen)
	_GDIPlus_BrushDispose($hBrushTrans)
	_GDIPlus_GraphicsDispose($hGfxScale)
	_GDIPlus_BitmapDispose($hBmpScale)
	_GDIPlus_BrushDispose($hBrush)
	_GDIPlus_GraphicsDispose($hGfxBuffer)
	_GDIPlus_BitmapDispose($hBmpBuffer)
	_GDIPlus_GraphicsDispose($hGraphics)
	_GDIPlus_Shutdown()
	Exit
EndFunc   ;==>_Exit

Func WM_PAINT($hWnd, $uMsgm, $wParam, $lParam)
	_GDIPlus_GraphicsDrawImage($hGraphics, $hBmpBuffer, 0, 0)
	Return 'GUI_RUNDEFMSG'
EndFunc   ;==>WM_PAINT

Func WM_ERASEBKGND($hWnd, $uMsgm, $wParam, $lParam)
	_GDIPlus_GraphicsDrawImage($hGraphics, $hBmpBuffer, 0, 0)
	Return True
EndFunc   ;==>WM_ERASEBKGND


Func ___DeBug($iError, $sAction)
	Switch $iError
		Case -1
			ConsoleWrite(@CRLF & "-" & $sAction & @CRLF)
		Case 0
			ConsoleWrite(@CRLF & "+" & $sAction & " - OK" & @CRLF)
		Case Else
			ConsoleWrite(@CRLF & "!" & $sAction & " - FAILED" & @CRLF)
			Exit
	EndSwitch
EndFunc   ;==>___DeBug